from fastapi import FastAPI, Depends, Response, HTTPException, status, Request
from typing import Dict, Any, List, Optional, Callable, Union, Tuple
from pydantic import BaseModel, EmailStr, Field
import uuid
import logging
from datetime import datetime, timedelta
from enum import Enum

from voidring import IndexedRocksDB
from .http import handle_errors, HttpMethod
from .models import User, UserRole, Result
from .tokens import TokensManager, TokenClaims, TokenBlacklist
from .users import UsersManager

def _set_auth_cookies(response: Response, access_token: str, logger: logging.Logger) -> None:
    """设置认证Cookie"""
    try:
        if access_token:
            response.set_cookie(
                key="access_token",
                value=access_token,
                httponly=True,
                secure=False,
                samesite="Lax",
                max_age=3600*24*30
            )
        else:
            response.delete_cookie("access_token")
        logger.info(f"设置 cookies: {access_token}")
    except Exception as e:
        logger.error(f"设置 cookies 时发生错误: {str(e)}")
        raise

def require_user(
    tokens_manager: TokensManager,
    require_roles: Union[UserRole, List[UserRole]] = None,
    update_access_token: bool = True,
    logger: logging.Logger = None
) -> Callable[[Request, Response], Dict[str, Any]]:
    """验证用户信息

    Args:
        tokens_manager: 令牌管理器
        require_roles: 要求的角色
    """

    async def verified_user(
        request: Request,
        response: Response,
    ) -> Dict[str, Any]:
        """验证用户信息

        如果要求角色，则需要用户具备所有指定的角色。
        """

        # 获取当前用户的访问令牌
        access_token = request.cookies.get("access_token")
        if not access_token:
            error = "令牌不存在，您必须登录获取刷新令牌和访问令牌"
            logger.error(error)
            raise HTTPException(
                status_code=status.HTTP_401_UNAUTHORIZED,
                detail=error
            )

        # 验证访问令牌
        verify_result = tokens_manager.verify_access_token(access_token)        
        if not verify_result.is_ok():
            error = f"令牌验证失败: {verify_result.error}"
            logger.error(error)
            raise HTTPException(
                status_code=status.HTTP_401_UNAUTHORIZED, 
                detail=error
            )

        token_claims = verify_result.data
        logger.debug(f"验证用户信息: {token_claims}")

        # 更新令牌到 response 的 cookie
        # JWT 的 encode 和 decode 通常基于对 CPU 可以忽略不计的 HMAC 计算
        if update_access_token:
            access_token = TokenClaims.create_access_token(**token_claims).jwt_encode()
            _set_auth_cookies(response, access_token, logger)
            logger.debug(f"设置令牌到 Cookie: {access_token}")

        # 如果要求所有角色，则需要用户具备指定的角色
        if require_roles and not UserRole.has_role(require_roles, token_claims['roles']):
            raise HTTPException(
                status_code=status.HTTP_403_FORBIDDEN,
                detail="权限不足。需要指定的角色。"
            )

        return token_claims

    return verified_user

def create_auth_endpoints(
    app: FastAPI,
    tokens_manager: TokensManager = None,
    users_manager: UsersManager = None,
    token_blacklist: TokenBlacklist = None,
    prefix: str="/api",
    logger: logging.Logger = None
) -> Dict[str, Tuple[HttpMethod, str, Callable]]:
    """创建认证相关的API端点
    
    Returns:
        Dict[str, Tuple[HttpMethod, str, Callable]]: 
            键为路由名称，
            值为元组 (HTTP方法, 路由路径, 处理函数)
    """

    logger = logging.getLogger(__name__)

    def _create_browser_device_id(request: Request) -> str:
        """为浏览器创建或获取设备ID
        
        优先从cookie中获取，如果没有则创建新的
        """
        existing_device_id = request.cookies.get("device_id")
        if existing_device_id:
            return existing_device_id
        
        user_agent = request.headers.get("user-agent", "unknown")
        os_info = "unknown_os"
        browser_info = "unknown_browser"
        
        if "Windows" in user_agent:
            os_info = "Windows"
        elif "Macintosh" in user_agent:
            os_info = "Mac"
        elif "Linux" in user_agent:
            os_info = "Linux"
        
        if "Chrome" in user_agent:
            browser_info = "Chrome"
        elif "Firefox" in user_agent:
            browser_info = "Firefox"
        elif "Safari" in user_agent and "Chrome" not in user_agent:
            browser_info = "Safari"
        
        return f"{os_info}_{browser_info}_{uuid.uuid4().hex[:8]}"

    class RegisterRequest(BaseModel):
        """注册请求"""
        username: str = Field(..., description="用户名")
        password: str = Field(..., description="密码")
        email: EmailStr = Field(..., description="邮箱")

    @handle_errors()
    async def register(request: RegisterRequest):
        """用户注册接口"""
        user = User(
            username=request.username,
            email=request.email,
            password_hash=User.hash_password(request.password),
        )
        result = users_manager.create_user(user)
        if result.is_ok():
            return result
        else:
            raise HTTPException(
                status_code=status.HTTP_400_BAD_REQUEST,
                detail=result.error
            )

    class LoginRequest(BaseModel):
        """登录请求
        支持用户从多个设备使用自动生成的设备ID同时登录。
        """
        username: str = Field(..., description="用户名")
        password: str = Field(..., description="密码")
        device_id: Optional[str] = Field(None, description="设备ID")

    @handle_errors()
    async def login(request: Request, response: Response, login_data: LoginRequest):
        """登录"""
        # 验证用户密码
        verify_result = users_manager.verify_password(
            username=login_data.username,
            password=login_data.password
        )
        
        if verify_result.is_fail():
            raise HTTPException(
                status_code=status.HTTP_401_UNAUTHORIZED,
                detail=verify_result.error or "认证失败"
            )
        
        user_info = verify_result.data
        logger.debug(f"登录结果: {user_info}")

        # 检查用户状态
        if user_info['is_locked']:
            raise HTTPException(
                status_code=status.HTTP_403_FORBIDDEN,
                detail="账户已锁定"
            )                
        if not user_info['is_active']:
            raise HTTPException(
                status_code=status.HTTP_403_FORBIDDEN,
                detail="账户未激活"
            )
            
        # 获取或创建设备ID
        device_id = login_data.device_id or _create_browser_device_id(request)

        # 更新设备刷新令牌
        tokens_manager.update_refresh_token(
            user_id=user_info['user_id'],
            username=user_info['username'],
            roles=user_info['roles'],
            device_id=device_id
        )
        logger.debug(f"更新设备刷新令牌: {device_id}")

        # 创建设备访问令牌
        result = _refresh_access_token(
            user_info=user_info,
            device_id=device_id,
            response=response
        )

        return result.data

    def _refresh_access_token(
        user_info: Dict[str, Any],
        device_id: str,
        response: Response,
    ):
        """刷新访问令牌"""
        result = tokens_manager.refresh_access_token(
            user_id=user_info['user_id'],
            device_id=device_id,
            username=user_info['username'],
            roles=user_info['roles']
        )
        logger.debug(f"创建设备访问令牌: {result}")
        if result.is_ok():
            access_token = TokenClaims.create_access_token(**result.data).jwt_encode()
            _set_auth_cookies(response, access_token=access_token, logger=logger)
        else:
            raise HTTPException(
                status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
                detail=result.error
            )
        return result

    @handle_errors()
    async def logout_device(
        request: Request,
        response: Response,
        token_claims: TokenClaims = Depends(require_user(tokens_manager, update_access_token=False, logger=logger))
    ):
        """退出在设备上的登录"""
        logger.debug(f"要注销的用户信息: {token_claims}")

        # 撤销当前设备的刷新令牌
        tokens_manager.revoke_refresh_token(
            user_id=token_claims['user_id'],
            device_id=token_claims['device_id']
        )
        
        # 撤销当前设备的访问令牌 - 加入黑名单
        tokens_manager.revoke_access_token(
            user_id=token_claims['user_id'],
            device_id=token_claims['device_id']
        )
        
        # 删除当前设备的cookie
        _set_auth_cookies(response, access_token=None, logger=logger)

        return {"message": "注销成功"}

    class ChangePasswordRequest(BaseModel):
        """修改密码请求"""
        current_password: str = Field(..., description="当前密码")
        new_password: str = Field(..., description="新密码")

    @handle_errors()
    async def change_password(
        change_password_form: ChangePasswordRequest,
        response: Response,
        token_claims: TokenClaims = Depends(require_user(tokens_manager, logger=logger))
    ):
        """修改密码"""
        result = users_manager.change_password(
            user_id=token_claims['user_id'],
            current_password=change_password_form.current_password,
            new_password=change_password_form.new_password
        )
        if result.is_ok():
            return result
        else:
            raise HTTPException(
                status_code=status.HTTP_400_BAD_REQUEST,
                detail=result.error
            )

    @handle_errors()
    async def get_user_profile(
        token_claims: TokenClaims = Depends(require_user(tokens_manager, logger=logger))
    ):
        """获取当前用户信息"""
        return token_claims

    class UpdateUserProfileRequest(BaseModel):
        """更新用户个人设置请求"""
        to_update: Dict[str, Any] = Field(..., description="用户个人设置")

    @handle_errors()
    async def update_user_profile(
        update_form: UpdateUserProfileRequest,
        response: Response,
        token_claims: TokenClaims = Depends(require_user(tokens_manager, logger=logger))
    ):
        """更新当前用户的个人设置"""
        result = users_manager.update_user(token_claims['user_id'], **update_form.to_update)
        if result.is_ok():
        # 更新设备访问令牌
            result = _refresh_access_token(
                user_info=result.data,
                device_id=token_claims['device_id'],
                response=response
            )
            if result.is_fail():
                raise HTTPException(
                    status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
                    detail=result.error
                )
            result.message = "用户信息更新成功"
            return result.data
        else:
            raise HTTPException(
                status_code=status.HTTP_400_BAD_REQUEST,
                detail=result.error
            )

    @handle_errors()
    async def check_blacklist(token_data: Dict[str, Any]):
        """检查令牌是否在黑名单中"""
        # 确保提供了必要字段
        user_id = token_data.get("user_id")
        device_id = token_data.get("device_id")
        
        if not user_id or not device_id:
            raise HTTPException(
                status_code=status.HTTP_400_BAD_REQUEST,
                detail="缺少必要的user_id或device_id字段"
            )
        
        # 直接使用user_id和device_id组合作为黑名单键
        token_id = f"{user_id}:{device_id}"
        
        # 检查是否在黑名单中
        is_blacklisted = token_blacklist.contains(token_id)
        return {"is_blacklisted": is_blacklisted}
    
    return [
        (HttpMethod.POST, f"{prefix}/auth/register", register),
        (HttpMethod.POST, f"{prefix}/auth/login", login),
        (HttpMethod.POST, f"{prefix}/auth/logout", logout_device),
        (HttpMethod.POST, f"{prefix}/auth/change-password", change_password),
        (HttpMethod.POST, f"{prefix}/auth/profile", update_user_profile),
        (HttpMethod.GET, f"{prefix}/auth/profile", get_user_profile),
        (HttpMethod.POST, f"{prefix}/token/blacklist/check", check_blacklist)
    ]
